/* gps_structures.h
 *
 * Command structure definitions
 *
 * This file can be used by Lua FFI.
 *
 */

#pragma pack(push)
#pragma pack(1)

enum { SIZE_GPS_RX_DATA = 128 };

typedef enum { GPS_TS_OFFSET = 1, GPS_GET_SYNC_TS, GPS_GET_TS,
               GPS_SETUP_SYNC_TS, GPS_TXDR_POWER_ON_5V, GPS_TXDR_POWER_ON_12V,
               GPS_TXDR_POWER_OFF, GPS_UART_SETUP,
               GPS_UART_RX_START, GPS_UART_RX_STOP,
               GPS_UART_TX_START, GPS_UART_TX_CHAR, GPS_UART_TX_STOP,
               GPS_EVENT_SETUP, GPS_EVENT_START, GPS_EVENT_STOP,
               GPS_FLOW_CONTROL, GPS_GET_CABLE_SENSE,
               GPS_LAST_COMMAND = 0x7FFFFFFF    /* Insures 32-bit enum */
} gps_cmd_t;

typedef struct {
    cmd_rate_parameters_s       baud_rate;
    uint8_t                     uart_setting[4];
} gps_cmd_uart_setup_s;

typedef struct {
    int64_t                     start_time;
    cmd_rate_parameters_s       heartbeat_interval;
    uint32_t                    phys_address;
    cmd_rate_parameters_s       baud_rate;
    uint8_t                     uart_setting[4];
} gps_cmd_rx_start_s;

typedef struct {
    uint32_t                phys_address;
} gps_cmd_tx_start_s;

typedef struct {
    char            txdata;
} gps_cmd_uart_tx_char_s;

typedef struct {
    uint8_t         cable_unplugged;
    uint8_t         cable_plugged;
    uint8_t         start_of_break;
    uint8_t         end_of_break;
    uint8_t         pps_rising;
    uint8_t         pps_falling;
    uint8_t         parity_error;
    uint8_t         frame_error;
} gps_cmd_event_vector_s;

typedef struct {
    uint8_t         rts_state;
    uint8_t         update_rts;
    uint8_t         reserved[2];
} gps_cmd_flow_control_s;

typedef struct {
    gps_cmd_t       cmd;
    uint32_t        cmd_id;
    union {
        cmd_ts_offset_s         ts_offset;
        gps_cmd_uart_setup_s    uart_setup;
        gps_cmd_rx_start_s      rx_start;
        gps_cmd_tx_start_s      tx_start;
        gps_cmd_uart_tx_char_s  tx_char;
        cmd_event_start_s       ev_start;
        gps_cmd_event_vector_s  events;
        gps_cmd_flow_control_s  flow_control;
    } prm;
} gps_cmd_s;

typedef struct {
    int64_t     ts;
    uint8_t    rxdata[SIZE_GPS_RX_DATA];
    uint8_t    count;
    uint8_t    reserved[3];
} rx_buffer_t;

typedef struct {
    char        txdata[64];
} tx_buffer_t;

typedef struct {
    int64_t                     ts;
    gps_cmd_event_vector_s      events;
} gps_ev_buffer_t;

#pragma pack(pop)
