local ffi = require 'ffi'
local lj_include = require 'lj_include'

lj_include('bits_types')

ffi.cdef[[
enum __rlimit_resource {
    RLIMIT_CPU = 0,
    RLIMIT_FSIZE = 1,
    RLIMIT_DATA = 2,
    RLIMIT_STACK = 3,
    RLIMIT_CORE = 4,
    __RLIMIT_RSS = 5,
    RLIMIT_NOFILE = 7,
    __RLIMIT_OFILE = RLIMIT_NOFILE,
    RLIMIT_AS = 9,
    __RLIMIT_NPROC = 6,
    __RLIMIT_MEMLOCK = 8,
    __RLIMIT_LOCKS = 10,
    __RLIMIT_SIGPENDING = 11,
    __RLIMIT_MSGQUEUE = 12,
    __RLIMIT_NICE = 13,
    __RLIMIT_RTPRIO = 14,
    __RLIMIT_RTTIME = 15,
    __RLIMIT_NLIMITS = 16,
    __RLIM_NLIMITS = __RLIMIT_NLIMITS
};
typedef __rlim_t rlim_t;
typedef __rlim64_t rlim64_t;
struct rlimit {
    rlim_t rlim_cur;
    rlim_t rlim_max;
};
struct rlimit64 {
    rlim64_t rlim_cur;
    rlim64_t rlim_max;
};
enum __rusage_who {
    RUSAGE_SELF = 0,
    RUSAGE_CHILDREN = -1,
    RUSAGE_THREAD = 1
};
]]

lj_include('bits_time')

ffi.cdef[[
struct rusage {
    struct timeval ru_utime;
    struct timeval ru_stime;
    union {
	long int ru_maxrss;
	__syscall_slong_t __ru_maxrss_word;
    };
    union {
	long int ru_ixrss;
	__syscall_slong_t __ru_ixrss_word;
    };
    union {
	long int ru_idrss;
	__syscall_slong_t __ru_idrss_word;
    };
    union {
	long int ru_isrss;
	__syscall_slong_t __ru_isrss_word;
    };
    union {
	long int ru_minflt;
	__syscall_slong_t __ru_minflt_word;
    };
    union {
	long int ru_majflt;
	__syscall_slong_t __ru_majflt_word;
    };
    union {
	long int ru_nswap;
	__syscall_slong_t __ru_nswap_word;
    };
    union {
	long int ru_inblock;
	__syscall_slong_t __ru_inblock_word;
    };
    union {
	long int ru_oublock;
	__syscall_slong_t __ru_oublock_word;
    };
    union {
	long int ru_msgsnd;
	__syscall_slong_t __ru_msgsnd_word;
    };
    union {
	long int ru_msgrcv;
	__syscall_slong_t __ru_msgrcv_word;
    };
    union {
	long int ru_nsignals;
	__syscall_slong_t __ru_nsignals_word;
    };
    union {
	long int ru_nvcsw;
	__syscall_slong_t __ru_nvcsw_word;
    };
    union {
	long int ru_nivcsw;
	__syscall_slong_t __ru_nivcsw_word;
    };
};
enum __priority_which {
    PRIO_PROCESS = 0,
    PRIO_PGRP = 1,
    PRIO_USER = 2
};
int prlimit(__pid_t __pid, enum __rlimit_resource __resource,
    const struct rlimit *__new_limit, struct rlimit *__old_limit);
int prlimit64(__pid_t __pid, enum __rlimit_resource __resource,
    const struct rlimit64 *__new_limit, struct rlimit64 *__old_limit);
]]

local defs = {
    RLIM_INFINITY = '~0',
    RLIM64_INFINITY = '~0',
    RLIM_SAVED_MAX = '~0',
    RLIM_SAVED_CUR = '~0',
    PRIO_MAX = '20',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

