local ffi = require 'ffi'
local lj_include = require 'lj_include'

ffi.cdef[[
struct timex {
    unsigned int modes;
    __syscall_slong_t offset;
    __syscall_slong_t freq;
    __syscall_slong_t maxerror;
    __syscall_slong_t esterror;
    int status;
    __syscall_slong_t constant;
    __syscall_slong_t precision;
    __syscall_slong_t tolerance;
    struct timeval time;
    __syscall_slong_t tick;
    __syscall_slong_t ppsfreq;
    __syscall_slong_t jitter;
    int shift;
    __syscall_slong_t stabil;
    __syscall_slong_t jitcnt;
    __syscall_slong_t calcnt;
    __syscall_slong_t errcnt;
    __syscall_slong_t stbcnt;
    int tai;
    int:32;
    int:32;
    int:32;
    int:32;
    int:32;
    int:32;
    int:32;
    int:32;
    int:32;
    int:32;
    int:32;
};
]]

local defs = {
    ADJ_OFFSET = '0x0001',
    ADJ_FREQUENCY = '0x0002',
    ADJ_MAXERROR = '0x0004',
    ADJ_ESTERROR = '0x0008',
    ADJ_STATUS = '0x0010',
    ADJ_TIMECONST = '0x0020',
    ADJ_TAI = '0x0080',
    ADJ_MICRO = '0x1000',
    ADJ_NANO = '0x2000',
    ADJ_TICK = '0x4000',
    ADJ_OFFSET_SINGLESHOT = '0x8001',
    ADJ_OFFSET_SS_READ = '0xa001',
    MOD_OFFSET = '0x0001',
    MOD_FREQUENCY = '0x0002',
    MOD_MAXERROR = '0x0004',
    MOD_ESTERROR = '0x0008',
    MOD_STATUS = '0x0010',
    MOD_TIMECONST = '0x0020',
    MOD_CLKB = '0x4000',
    MOD_CLKA = '0x8001',
    MOD_TAI = '0x0080',
    MOD_MICRO = '0x1000',
    MOD_NANO = '0x2000',
    STA_PLL = '0x0001',
    STA_PPSFREQ = '0x0002',
    STA_PPSTIME = '0x0004',
    STA_FLL = '0x0008',
    STA_INS = '0x0010',
    STA_DEL = '0x0020',
    STA_UNSYNC = '0x0040',
    STA_FREQHOLD = '0x0080',
    STA_PPSSIGNAL = '0x0100',
    STA_PPSJITTER = '0x0200',
    STA_PPSWANDER = '0x0400',
    STA_PPSERROR = '0x0800',
    STA_CLOCKERR = '0x1000',
    STA_NANO = '0x2000',
    STA_MODE = '0x4000',
    STA_CLK = '0x8000',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

