--! \file

local m = {}

--[[
-- Assorted information from the MX devices that we don't use
local xpaths = {
'/Amplifier/Connector#channel_num/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/CalPeriod',
'/Amplifier/Connector#channel_num/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/SignalName',
'/Amplifier/Connector#channel_num/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/OriginOfName',
'/Amplifier/Connector#channel_num/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/Decimals',
'/Amplifier/Connector#channel_num/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/SignalFormat',
'/Amplifier/Connector#channel_num/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/Output/Description',
'/Amplifier/Connector#channel_num/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/Output/DaqAvailable',
'/Amplifier/Connector#channel_num/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/Output/RtAvailable',
}
]]

-- This is the mapping of JSON style setup parameters to XPATHs for the device
m.json2xpath = {
    sample_rate = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/OutputRate',

    sensor_input = {
        mode = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Sensortype/@choice',
        max = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Sensortype[@choice="#sensor_input_mode#"]/#sensor_input_mode#/MaxElectrical',
        min = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Sensortype[@choice="#sensor_input_mode#"]/#sensor_input_mode#/MaxElectrical',
    },

    calibration = {
        -- Define source units
        define_source_units = {
            units = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/PhysUnit',
        },
        -- Scaling unit
        sensor_scaling = {
            mode = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype/@choice',
            electrical_zero = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="ZeroSpan"]/ZeroSpan/ElecZero',
            electrical_span = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="ZeroSpan"]/ZeroSpan/ElecSpan',
            physical_span = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="ZeroSpan"]/ZeroSpan/PhysSpan',
            num_points = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="LinTable"]/LinTable/Count',
            electrical = {
                [1] = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="LinTable"]/LinTable/Point[@address="1"]/X',
                [2] = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="LinTable"]/LinTable/Point[@address="2"]/X',
            },
            physical = {
                [1] = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="LinTable"]/LinTable/Point[@address="1"]/Y',
                [2] = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="LinTable"]/LinTable/Point[@address="2"]/Y',
            },
            slope = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="Gain"]/Gain/m',
            intercept = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="Gain"]/Gain/b',
            gage_factor = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="Straingage"]/Straingage/kFactor',
            bridge_factor = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="Straingage"]/Straingage/Bridgefactor',
            resistance = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Scaling/Scalingtype[@choice="Thermocouple"]/Thermocouple/Resistance',
            units = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/PhysUnit',
            _units = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/PhysUnitUsr',
        },
        -- Output unit conversion unit
        units_conversion = {
            _units = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/OutputUnit/UserText',
            units = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/OutputUnit/UnitId',
            intercept = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/OutputUnit/Offset',
            slope = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/OutputUnit/Factor',
        },
        -- Zero balance unit
        rezero = {
--            target = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/ZeroOffset/Target',
            intercept = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/ZeroOffset/Offset',
--            mode = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/ZeroOffset/Inhibit',
        },
    },

    drift_compensation = {
        mode = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Sensortype[@choice="#sensor_input_mode#"]/#sensor_input_mode#/DriftCompensation/Request/@choice',
        repetition_time = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Sensortype[@choice="#sensor_input_mode#"]/#sensor_input_mode#/DriftCompensation/Request[@choice="#mode#"]/#mode#/RepetitionTime',
    },

    teds_sensor = {
        id = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/SensorId',
        name = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/SensorName',
        manufacturer = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/ManuName',
        serial_number = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/SerialNumber',
        type = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/TypeName',
        calibration_date = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/CalDate',
        calibration_initials = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/CalInitials',
        calibration_period = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/CalPeriod',
        location = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/Location',
        user_id = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Identification/UserDefId',
        use_teds = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/TedsOptions/UseTeds',
        bin_data = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/TedsOptions/BinData',
    },

    _digital_filter_choice = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/FilterType/@choice',
    digital_filter_type = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/FilterType/Lowpass/FiltCharact',
    digital_filter_attenuation_frequency = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/FilterType/Lowpass/FiltFreq',

    _reference = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/SyncSignal[@address="1"]/Output/SignalReference',
}

-- List of parameters in a channel setup that we do NOT want sent off to the MX
-- device
m.ignore_list = {
    _reference = true,
}

-- List of parameters returned by MX device that we don't verify after setting
-- up a channel.
m.dont_verify_list = {
    calibration = {
        {
            num_points = true,
        }
    },
}

return m
