local ffi = require 'ffi'
local C = ffi.C
local ts = require 'timestamps'
local config = require 'system_config'

if config.device_type == 'cx23r' then
    ffi.cdef[[
        struct tm {
            int tm_sec;
            int tm_min;
            int tm_hour;
            int tm_mday;
            int tm_mon;
            int tm_year;
            int tm_wday;
            int tm_yday;
            int tm_isdst;
        };
    ]]
end

local m = { }

function m.init(enablepps)
    m.date = ffi.new('struct tm')
    m.date.tm_year = 0
    m.subseconds = 0
end

function m.merge_hhmmss(time)
    local h, min, s, frac = time:match("^(%d%d)(%d%d)(%d%d)(.+)$")

    m.date.tm_hour = tonumber(h)
    if tonumber(h) < m.date.tm_hour then
        m.date.tm_mday = m.date.tm_mday + 1
    end
    m.date.tm_min = tonumber(min)
    m.date.tm_sec = tonumber(s)
    m.subseconds = tonumber(frac)
end

function m.merge_ddmmyy(date)
    local d, mon, y = date:match("^(%d%d)(%d%d)(%d%d)$")
    m.date.tm_year = (2000 + tonumber(y)) - 1900
    m.date.tm_mon = tonumber(mon) - 1
    m.date.tm_mday = tonumber(d)
end

local cumdays = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334}

function m.mkgmtime()
    local year = ffi.new('int', 1900 + m.date.tm_year)
    local result = ffi.new('time_t')

    result = (year - 1970) * 365 + cumdays[m.date.tm_mon + 1]
    result = result + (year - 1968) / 4
    result = result - (year - 1900) / 100
    result = result + (year - 1600) / 400
    if ((year % 4) == 0 and ((year % 100) ~= 0 or (year % 400) == 0) and
        (m.date.tm_mon < 2)) then result = result - 1 end
    result = result + (m.date.tm_mday - 1)
    result = result * 24
    result = result + m.date.tm_hour
    result = result * 60
    result = result + m.date.tm_min
    result = result * 60
    result = result + m.date.tm_sec
    return tonumber(result) + m.subseconds
end

function m.nmea_process(sentence)
    if not sentence:find("^%$GPRMC,") then return end
    local time, status, date =
        sentence:match("^%$GPRMC,(.-),(.-),.+,(%d%d%d%d%d%d),.+$")
    if time and status and date and status ~= 'V' and
        (time:find("%.[0,2,4,6,8]0$") or time:find("%.[0,2,4,6,8]$")) then
        m.merge_hhmmss(time)
        m.merge_ddmmyy(date)
        return m.mkgmtime()
    end
end

return m
