require 'strict'
local ffi = require 'ffi'
local C = ffi.C
local frame = require 'test_engine/frame'
local config = require 'system_config'
local libte = ffi.load(config.test_engine_library)

if config.test_engine_library == '/usr/lib/libte-debug.so' then
ffi.cdef[[
    struct pipe {
        struct pipe     *next;          // Next frame in list
        struct pipe     *prev;          // Previous frame in list
        int64_t         sample_count;   // Number of samples in pipe
        uint32_t        frame_seq;      // Next frame sequence number (debug)
        uint32_t        expected_frame_seq;
        struct frame    *frame_list;    // Frame list
        struct frame    *last_frame;    // Last frame in frame list
    };
]]
else
ffi.cdef[[
    struct pipe {
        struct pipe     *next;          // Next frame in list
        struct pipe     *prev;          // Previous frame in list
        int64_t         sample_count;   // Number of samples in pipe
        struct frame    *frame_list;    // Frame list
        struct frame    *last_frame;    // Last frame in frame list
    };
]]
end

ffi.cdef[[

    void *calloc(size_t nmemb, size_t size);
    void free(void *ptr);

    struct pipe *pipeAttachPipe(struct pipe **pipe_list);
    void pipeDetachPipe(struct pipe **pipe_list, struct pipe *pipe);
    void pipeAddFrame(struct pipe *pipe, struct frame *f);
    struct frame *pipeGetFrame(struct pipe *pipe);
    void pipeFlushFrames(struct pipe *pipe);
    int pipeHasFrames(struct pipe *pipe);
    struct pipe **pipeNewPipeList(void);
]]

return {
    attach_pipe = libte.pipeAttachPipe,
    detach_pipe = libte.pipeDetachPipe,
    add_frame = libte.pipeAddFrame,
    get_frame = libte.pipeGetFrame,
    flush_frames = libte.pipeFlushFrames,
    has_frames = function(p) return libte.pipeHasFrames(p) > 0 end,
    new_pipe_list = libte.pipeNewPipeList,
}
